

//If simulation, please define SIM
// `define SIM 

module flash_test
(
	input             clk          ,
	input             rst_n        ,
	output reg        right_o      ,
	
	output reg [31:0] haddr_reg    ,//AHB address
	input      [31:0] hrdata_reg   ,//AHB read data
	output            hreadyin_reg ,// ?
	input             hreadyout_reg,//from slave
	input       [1:0] hresp_reg    ,//slave transfer response,OKAY, ERROR, RETRY, SPLIT
	output            hsel_reg     ,//1:select 
	output reg  [1:0] htrans_reg   ,//transfer type, NSEQ 10, SEQ 11, IDLE 00, BUSY 01
	output reg        hwrite_reg   ,//1:write,  0:read
	output reg [31:0] hwdata_reg    //AHB write data
);

//============================================================================
localparam	IDLE                 =  5'd00; 
localparam	SET_TRANS_FMT        =  5'd01; 

localparam	SE_WREN_SET_CTRL     =  5'd07; //Sector Erase
localparam	SE_WREN_SET_CMD      =  5'd08; 
localparam	SE_SET_TRANS_CTRL    =  5'd09; 
localparam	SE_SET_ADDR          =  5'd10;  
localparam	SE_SET_CMD           =  5'd11;  

localparam	WR_WAIT_RDY          =  5'd15;   
localparam	WR_WREN_SET_CTRL     =  5'd16; //write
localparam	WR_WREN_SET_CMD      =  5'd17; 
localparam	WR_SET_CTRL          =  5'd18; 
localparam	WR_SET_DATA          =  5'd19; 	
localparam	WR_SET_ADDR          =  5'd20;
localparam	WR_SET_CMD           =  5'd21;

localparam	RD_WAIT_RDY          =  5'd22;  
localparam	RD_SET_CTRL          =  5'd23; //read	
localparam	RD_SET_ADDR          =  5'd24;
localparam	RD_SET_CMD           =  5'd25;
localparam	RD_GET_READ_DATA     =  5'd26;

//-----------------------------------------


`ifdef SIM
    localparam	SE_CMD_TIME        =  32'd1_000_000;
    localparam	WREN_CMD_TIME      =  12'd63;
    localparam	WR_CMD_TIME        =  16'd20000; //50MHz, 6000->120us  //25MHz, 3000->120us
    localparam	RD_CMD_TIME        =  12'd2000;
`else
    localparam	SE_CMD_TIME        =  32'd50_000_000;
    localparam	WREN_CMD_TIME      =  12'd63;
    localparam	WR_CMD_TIME        =  16'd3000; //50MHz, 6000->120us  //25MHz, 3000->120us
    localparam	RD_CMD_TIME        =  12'd255;
`endif
	


// One block max address is 00FFFFH
// One sector max address is 000FFFH
`ifdef SIM
    localparam  CHECK_MAX_ADDRESS    =  32'h0000001f;
`else
    localparam  CHECK_MAX_ADDRESS    =  32'h0000_1fff;
`endif

//============================================================================
reg  [31:0] start_cnt;
reg         start_i;
reg         start_i_d1;
reg         start_i_d2;
reg  [31:0] wr_addr_reg;
wire [31:0] wr_data_reg;

reg  [31:0] rd_addr_reg;

reg  [4:0]  current_state;
reg  [4:0]  next_state;

reg  [7:0]  cnt;

reg  [31:0] se_cmd_cnt;
reg  [11:0] wren_cmd_cnt;
reg  [15:0] wr_cmd_cnt;
reg  [11:0] rd_cmd_cnt;

reg  [31:0] rd_data;

reg         error_flag;

reg  [7:0]  wrdata_cnt;
reg  [7:0]  rddata_cnt;

//=======================================================================
assign hsel_reg  = 1'b1;
assign hreadyin_reg = 1'b1;

//=======================================================================
always@(posedge clk or negedge rst_n)
begin
    if(!rst_n)
        start_cnt <= 32'd0;
    else if(start_cnt >= 32'hffff_ffff)
        start_cnt <= start_cnt;
    else
        start_cnt <= start_cnt + 1'b1;
end

always@(posedge clk or negedge rst_n)
begin
    if(!rst_n)
        start_i <= 1'b1;
    else if(start_cnt == 32'd2_000_000) //100ms
        start_i <= 1'b0;
    else
        start_i <= start_i;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		begin
			start_i_d1 <= 1'b0 ;
			start_i_d2 <= 1'b0 ;
		end
	else
		begin
			start_i_d1 <= start_i    ;
			start_i_d2 <= start_i_d1 ;
		end
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		wr_addr_reg <= 32'd0;
	else if(start_i_d2 && !start_i_d1) //falling edge
		wr_addr_reg <= 32'd0;
	else if(current_state == WR_SET_CMD && wr_cmd_cnt == WR_CMD_TIME)
		wr_addr_reg <= wr_addr_reg + 1'b1;
	else
		wr_addr_reg <= wr_addr_reg;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		rd_addr_reg <= 32'd0;
	else if(start_i_d2 && !start_i_d1) //falling edge
		rd_addr_reg <= 32'd0;
	else if(current_state == RD_GET_READ_DATA && cnt == 8'd7)
		rd_addr_reg <= rd_addr_reg + 1'b1;
	else
		rd_addr_reg <= rd_addr_reg;
end

//=========================================================
//write into flash data
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		wrdata_cnt <= 8'd0;
	else if(current_state == WR_SET_ADDR && cnt == 8'd7)
		wrdata_cnt <= wrdata_cnt + 1'b1;
	else
		wrdata_cnt <= wrdata_cnt;
end

assign wr_data_reg = wrdata_cnt;//32'h0000_003c;

//=======================================================================
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		cnt <= 8'd0;
	else if(hreadyout_reg)
		begin
			if(cnt == 8'd7)
				cnt <= 8'd0;
			else if(current_state == RD_SET_CMD || 
			        current_state == WR_SET_CMD ||
					current_state == WR_WREN_SET_CMD ||
					current_state == SE_WREN_SET_CMD || current_state == SE_SET_CMD)
				cnt <= 8'd0;
			else
				cnt <= cnt + 1'b1;
		end
	else
		cnt <= cnt;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		rd_cmd_cnt <= 12'd0;
	else if(current_state == RD_SET_CMD)
		begin
			if(rd_cmd_cnt >= RD_CMD_TIME)
				rd_cmd_cnt <= RD_CMD_TIME;
			else if(hreadyout_reg)
				rd_cmd_cnt <= rd_cmd_cnt + 1'b1;
			else
				rd_cmd_cnt <= rd_cmd_cnt;
		end
	else
		rd_cmd_cnt <= 12'd0;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		wr_cmd_cnt <= 16'd0;
	else if(current_state == WR_SET_CMD)
		begin
			if(wr_cmd_cnt >= WR_CMD_TIME)
				wr_cmd_cnt <= WR_CMD_TIME;
			else if(hreadyout_reg)
				wr_cmd_cnt <= wr_cmd_cnt + 1'b1;
			else
				wr_cmd_cnt <= wr_cmd_cnt;
		end
	else
		wr_cmd_cnt <= 16'd0;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		wren_cmd_cnt <= 12'd0;
	else if(current_state == WR_WREN_SET_CMD ||
			current_state == SE_WREN_SET_CMD)
		begin
			if(wren_cmd_cnt >= WREN_CMD_TIME)
				wren_cmd_cnt <= WREN_CMD_TIME;
			else if(hreadyout_reg)
				wren_cmd_cnt <= wren_cmd_cnt + 1'b1;
			else
				wren_cmd_cnt <= wren_cmd_cnt;
		end
	else
		wren_cmd_cnt <= 12'd0;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		se_cmd_cnt <= 32'd0;
	else if(current_state == SE_SET_CMD)
		begin
			if(se_cmd_cnt >= SE_CMD_TIME)
				se_cmd_cnt <= SE_CMD_TIME;
			else if(hreadyout_reg)
				se_cmd_cnt <= se_cmd_cnt + 1'b1;
			else
				se_cmd_cnt <= se_cmd_cnt;
		end
	else
		se_cmd_cnt <= 32'd0;
end

//=======================================================================
//FSM Control
//-----------------------------------------------
//(1)State change
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		current_state  <= IDLE;
	else
		current_state  <= next_state;
end

//-----------------------------------------------
//(2)State change condition
always@(*)
begin
	if(!rst_n)
		next_state  <= IDLE;
	else
		begin
			case(current_state)
				IDLE:
					begin
						if(start_i_d2 && !start_i_d1) //falling edge
							next_state <= SET_TRANS_FMT;
						else
							next_state <= IDLE;
					end
				SET_TRANS_FMT: 
					begin
						if(cnt == 8'd7)
							next_state <= SE_WREN_SET_CTRL;
						else
							next_state <= SET_TRANS_FMT;
					end
				
                    
                    
				SE_WREN_SET_CTRL:
					begin
						if(cnt == 8'd7)
							next_state <= SE_WREN_SET_CMD;
						else
							next_state <= SE_WREN_SET_CTRL;
					end
				SE_WREN_SET_CMD:
					begin
						if(wren_cmd_cnt == WREN_CMD_TIME)
							next_state <= SE_SET_TRANS_CTRL;
						else
							next_state <= SE_WREN_SET_CMD;
					end
				SE_SET_TRANS_CTRL:
					begin
						if(cnt == 8'd7)
							next_state <= SE_SET_ADDR;
						else
							next_state <= SE_SET_TRANS_CTRL;
					end
				SE_SET_ADDR:
					begin
						if(cnt == 8'd7)
							next_state <= SE_SET_CMD;
						else
							next_state <= SE_SET_ADDR;
					end
				SE_SET_CMD:
					begin
						if(se_cmd_cnt == SE_CMD_TIME)
							next_state <= WR_WAIT_RDY;
						else
							next_state <= SE_SET_CMD;
					end
                    
					
				WR_WAIT_RDY: 
					begin
						if(cnt == 8'd7 && wr_addr_reg<=CHECK_MAX_ADDRESS)
							next_state <= WR_WREN_SET_CTRL;
						else if(cnt == 8'd7 && wr_addr_reg>CHECK_MAX_ADDRESS)
							next_state <= RD_WAIT_RDY;
						else
							next_state <= WR_WAIT_RDY;
					end
				WR_WREN_SET_CTRL: //write start
					begin
						if(cnt == 8'd7)
							next_state <= WR_WREN_SET_CMD;
						else
							next_state <= WR_WREN_SET_CTRL;
					end
				WR_WREN_SET_CMD:
					begin
						if(wren_cmd_cnt == WREN_CMD_TIME)
							next_state <= WR_SET_CTRL;
						else
							next_state <= WR_WREN_SET_CMD;
					end	
				WR_SET_CTRL:
					begin
						if(cnt == 8'd7)
							next_state <= WR_SET_DATA;
						else
							next_state <= WR_SET_CTRL;
					end
				WR_SET_DATA:
					begin
						if(cnt == 8'd7)
							next_state <= WR_SET_ADDR;
						else
							next_state <= WR_SET_DATA;
					end
				WR_SET_ADDR:
					begin
						if(cnt == 8'd7)
							next_state <= WR_SET_CMD;
						else
							next_state <= WR_SET_ADDR;
					end
				WR_SET_CMD:
					begin
						if(wr_cmd_cnt == WR_CMD_TIME)
							next_state <= WR_WAIT_RDY;//RD_SET_CTRL;//WR_WAIT_RDY 
						else
							next_state <= WR_SET_CMD;
					end
					
				
				RD_WAIT_RDY: 
					begin
						if(cnt == 8'd7 && rd_addr_reg<=CHECK_MAX_ADDRESS)
							next_state <= RD_SET_CTRL;
						else if(cnt == 8'd7 && rd_addr_reg>CHECK_MAX_ADDRESS)
							next_state <= IDLE;
						else
							next_state <= RD_WAIT_RDY;
					end
				RD_SET_CTRL: //read start
					begin
						if(cnt == 8'd7)
							next_state <= RD_SET_ADDR;
						else
							next_state <= RD_SET_CTRL;
					end
				RD_SET_ADDR:
					begin
						if(cnt == 8'd7)
							next_state <= RD_SET_CMD;
						else
							next_state <= RD_SET_ADDR;
					end
				RD_SET_CMD:
					begin
						if(rd_cmd_cnt == RD_CMD_TIME)
							next_state <= RD_GET_READ_DATA;
						else
							next_state <= RD_SET_CMD;
					end
				RD_GET_READ_DATA:
					begin
						if(cnt == 8'd7)
							next_state <= RD_WAIT_RDY;
						else
							next_state <= RD_GET_READ_DATA;
					end
				
				default:
					begin
						next_state <= IDLE;
					end
			endcase
		end
end

//-----------------------------------------------
//(3)State output
always@(posedge clk or negedge rst_n)//(*)
begin
	if(!rst_n)
		begin
			htrans_reg       <= 2'b00          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
			haddr_reg        <= 32'h0000_0000  ;
			hwrite_reg       <= 1'b0           ;//1:write,  0:read
			hwdata_reg       <= 32'h0000_0000  ;
		end
	else
		begin
			case(next_state)//如将 next_state 改为 current_state，会使输出延时1拍/////
				IDLE:
					begin
						htrans_reg       <= 2'b00          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
						haddr_reg        <= 32'h0000_0000  ;
						hwrite_reg       <= 1'b0           ;//1:write,  0:read
						hwdata_reg       <= 32'h0000_0000  ;
					end
				
				SET_TRANS_FMT: 
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0010  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0002_0780  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				
                
				SE_WREN_SET_CTRL:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0020  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h4700_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				SE_WREN_SET_CMD:
					begin
						if(wren_cmd_cnt==12'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0024  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(wren_cmd_cnt==12'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0006  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end	
				SE_SET_TRANS_CTRL:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0020  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h6700_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end	
				SE_SET_ADDR:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0028  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				SE_SET_CMD:
					begin
						if(se_cmd_cnt==32'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0024  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(se_cmd_cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0020  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
					
					
					
				WR_WAIT_RDY:
					begin
						htrans_reg       <= 2'b00          ;
						haddr_reg        <= 32'h0000_0000  ;
						hwrite_reg       <= 1'b0           ;
						hwdata_reg       <= 32'h0000_0000  ;
					end
				WR_WREN_SET_CTRL:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0020  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h4700_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				WR_WREN_SET_CMD:
					begin
						if(wren_cmd_cnt==12'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0024  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(wren_cmd_cnt==12'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0006  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				WR_SET_CTRL:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0020  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h6100_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				WR_SET_DATA:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_002C  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= wr_data_reg    ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				WR_SET_ADDR:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0028  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= wr_addr_reg    ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				WR_SET_CMD:
					begin
						if(wr_cmd_cnt==16'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0024  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(wr_cmd_cnt==16'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0002  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				
				
				
				RD_WAIT_RDY:
					begin
						htrans_reg       <= 2'b00          ;
						haddr_reg        <= 32'h0000_0000  ;
						hwrite_reg       <= 1'b0           ;
						hwdata_reg       <= 32'h0000_0000  ;
					end
				RD_SET_CTRL: //read
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0020  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h6200_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				RD_SET_ADDR:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0028  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(cnt==8'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= rd_addr_reg    ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				RD_SET_CMD:
					begin
						if(rd_cmd_cnt==12'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_0024  ;
								hwrite_reg       <= 1'b1           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else if(rd_cmd_cnt==12'd2)
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0003  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
				RD_GET_READ_DATA:
					begin
						if(cnt==8'd1)
							begin
								htrans_reg       <= 2'b10          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
								haddr_reg        <= 32'h0000_002C  ;
								hwrite_reg       <= 1'b0           ;//1:write,  0:read
								hwdata_reg       <= 32'h0000_0000  ;
							end
						else
							begin
								htrans_reg       <= 2'b00          ;
								haddr_reg        <= 32'h0000_0000  ;
								hwrite_reg       <= 1'b0           ;
								hwdata_reg       <= 32'h0000_0000  ;
							end
					end
					
				
				default:
					begin
						htrans_reg       <= 2'b00          ;//NSEQ 10, SEQ 11, IDLE 00, BUSY 01
						haddr_reg        <= 32'h0000_0000  ;
						hwrite_reg       <= 1'b0           ;//1:write,  0:read
						hwdata_reg       <= 32'h0000_0000  ;
					end
			endcase
		end
end

//read data
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		rd_data        <= 32'h0000_0000  ;
	else
		begin
			case(next_state)//如将 next_state 改为 current_state，会使输出延时1拍///// 
				RD_GET_READ_DATA:
					begin
						if(cnt==8'd4)
							rd_data        <= hrdata_reg  ;
						else
							rd_data        <= rd_data  ;
					end
				default:
						rd_data        <= rd_data  ;
			endcase
		end
end

//=========================================================
//data used to compare with read data
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		rddata_cnt <= 8'd0;
	else if(current_state == RD_GET_READ_DATA && cnt == 8'd7)
		rddata_cnt <= rddata_cnt + 1'b1;
	else
		rddata_cnt <= rddata_cnt;
end

//==========================================================
always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		error_flag <= 1'b0;
	else if(start_i_d2 && !start_i_d1)
		error_flag <= 1'b0;
	else if(current_state == RD_GET_READ_DATA && cnt==8'd6 && rd_data!=rddata_cnt)
		error_flag <= 1'b1;
	else
		error_flag <= error_flag;
end

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		right_o <= 1'b0;
	else if(start_i_d2 && !start_i_d1)
		right_o <= 1'b0;
	else if(error_flag)
		right_o <= 1'b0;
	else if(current_state == RD_GET_READ_DATA && cnt==8'd6 && rd_data==rddata_cnt)
		right_o <= 1'b1;
	else
		right_o <= right_o;
end

// always@(posedge clk or negedge rst_n)
// begin
	// if(!rst_n)
		// right_o <= 1'b0;
	// else if(current_state == RD_GET_READ_DATA && cnt==8'd7 && rd_data==wr_data_reg)
		// right_o <= 1'b1;
	// else if(current_state == RD_GET_READ_DATA && cnt==8'd7 && rd_data==!wr_data_reg)
		// right_o <= 1'b0;
	// else
		// right_o <= right_o;
// end

endmodule
