

module flash_test_top
(
	input        I_clk      ,//27Mhz
	input        I_rst_n    ,//RST_N
	output [1:0] O_led      ,
	output       O_ck       ,
	output       O_cs_n     ,
	inout        IO_hold_n  ,//IO3
    inout        IO_wp_n    ,//IO2
	inout        IO_do      ,//IO1
	inout        IO_di       //IO0
);

//===========================================
reg  [31:0] run_cnt;
wire        running;

// ahb reg 
wire [31:0] haddr_reg;
wire [31:0] hrdata_reg;
wire       	hreadyin_reg;
wire       	hreadyout_reg;
wire [1:0] 	hresp_reg;
wire       	hsel_reg;
wire [1:0] 	htrans_reg;
wire       	hwrite_reg;
wire [31:0] hwdata_reg;

wire        right_o;

//===================================================
//LED test
always @(posedge I_clk or negedge I_rst_n) //I_clk
begin
	if(!I_rst_n)
		run_cnt <= 32'd0;
	else if(run_cnt >= 32'd27_000_000)
		run_cnt <= 32'd0;
	else
		run_cnt <= run_cnt + 1'b1;
end

assign  running = (run_cnt < 32'd13_500_000) ? 1'b1 : 1'b0;

assign  O_led[0] = running;
assign  O_led[1] = ~right_o;

//==================================================
//SPI Flash Control
flash_test flash_test_inst      
(
    .clk          (I_clk        ),//I_clk
    .rst_n        (I_rst_n      ),
	.right_o      (right_o      ),
    .haddr_reg    (haddr_reg    ), //ahb reg mode
 	.hrdata_reg   (hrdata_reg   ),
 	.hreadyin_reg (hreadyin_reg ),
 	.hreadyout_reg(hreadyout_reg),
 	.hresp_reg    (hresp_reg    ),
 	.hsel_reg     (hsel_reg     ),
 	.htrans_reg   (htrans_reg   ),
 	.hwdata_reg   (hwdata_reg   ),
 	.hwrite_reg   (hwrite_reg   )
);   

//===================================================
//SPI Flash Controller
SPI_Flash_Interface_Lite_Top SPI_Flash_Interface_Lite_Top_inst
(
    .I_hclk         (I_clk        ), //I_clk
	.I_hresetn      (I_rst_n      ),
    .I_haddr_reg    (haddr_reg    ),
 	.O_hrdata_reg   (hrdata_reg   ),
 	.I_hreadyin_reg (hreadyin_reg ),
 	.O_hreadyout_reg(hreadyout_reg),
 	.O_hresp_reg    (hresp_reg    ),
 	.I_hsel_reg     (hsel_reg     ),
 	.I_htrans_reg   (htrans_reg   ),
 	.I_hwdata_reg   (hwdata_reg   ),
 	.I_hwrite_reg   (hwrite_reg   ),
    
    .O_flash_ck     (O_ck         ),
	.O_flash_cs_n   (O_cs_n       ),
    .IO_flash_hold_n(IO_hold_n    ),//IO3
    .IO_flash_wp_n  (IO_wp_n      ),//IO2
	.IO_flash_do    (IO_do        ),//IO1
	.IO_flash_di    (IO_di        ) //IO0
);

endmodule