//---------------------------------------------------------------------
// File name  : tb.v
// Module name: tb
// Created by : 
// ---------------------------------------------------------------------
// Release history
// VERSION |  Date        | AUTHOR     |    DESCRIPTION
// ---------------------------------------------------------------------
//  1.0    |  1-June-2020 |            |     initial 
// ---------------------------------------------------------------------

`timescale 1ns / 1ps   

module tb();
//========================================================
//parameters
parameter MAIN_CLK_FREQ       = 25.000;//clock frequency, unit: MHz
parameter MAIN_CLK_PERIOD     = 1000.0/MAIN_CLK_FREQ; //unit: ns

   
parameter WAIT_TIME           = 200000000;

//=======================================================
reg  clk   ;      
reg  rst_n ;

wire right_o     ;

wire spi_clk_o     ;
wire spi_cs_n_o    ;
wire spi_hold_n_io ;
wire spi_wp_n_io   ;
wire spi_so_io     ;
wire spi_si_io     ;

// ahb reg 
wire [31:0] haddr_reg;
wire [31:0] hrdata_reg;
wire        hreadyin_reg;
wire        hreadyout_reg;
wire  [1:0] hresp_reg;
wire        hsel_reg;
wire  [1:0] htrans_reg;
wire        hwrite_reg;
wire [31:0] hwdata_reg;


//========================================================
GSR GSR(.GSRI(1'b1));

//==============================================  
initial begin
  $fsdbDumpfile("tb.fsdb");
  $fsdbDumpvars;
end

//============================================== 
//rst_n
initial
  begin
    rst_n=1'b0;
    
    #25;
    rst_n=1'b1;
	
	#WAIT_TIME;
    $finish;
end

//=====================================================
//clk
initial
begin
	clk = 1'b0;
end

always  #(MAIN_CLK_PERIOD/2.0) clk = ~clk;


//==================================================
//SPI Flash Control
flash_test flash_test_inst      
(
    .clk          (clk),
    .rst_n        (rst_n),
	.right_o      (right_o ),
	
    .haddr_reg    (haddr_reg), //ahb reg mode
 	.hrdata_reg   (hrdata_reg),
 	.hreadyin_reg (hreadyin_reg),
 	.hreadyout_reg(hreadyout_reg),
 	.hresp_reg    (hresp_reg),
 	.hsel_reg     (hsel_reg),
 	.htrans_reg   (htrans_reg),
 	.hwdata_reg   (hwdata_reg),
 	.hwrite_reg   (hwrite_reg)
);

//===================================================
//SPI Flash Controller
SPI_Flash_Interface_Lite_Top SPI_Flash_Interface_Lite_Top_inst
(
    .I_hclk         (clk  ), 
	.I_hresetn      (rst_n),
    .I_haddr_reg    (haddr_reg),//ahb reg mode 
 	.O_hrdata_reg   (hrdata_reg),
 	.I_hreadyin_reg (hreadyin_reg),
 	.O_hreadyout_reg(hreadyout_reg),
 	.O_hresp_reg    (hresp_reg),
 	.I_hsel_reg     (hsel_reg),
 	.I_htrans_reg   (htrans_reg),
 	.I_hwdata_reg   (hwdata_reg),
 	.I_hwrite_reg   (hwrite_reg),

    .O_flash_ck     (spi_clk_o    ),
	.O_flash_cs_n   (spi_cs_n_o   ),
    .IO_flash_hold_n(spi_hold_n_io),//IO3
    .IO_flash_wp_n  (spi_wp_n_io  ),//IO2
	.IO_flash_do    (spi_so_io    ),//IO1
	.IO_flash_di    (spi_si_io    ) //IO0
);

//==============================================
//flash_model
W25Q32JVxxIM W25Q32JVxxIM_inst 
(
    .CSn      (spi_cs_n_o   ), 
    .CLK      (spi_clk_o    ), 
    .DIO      (spi_si_io    ), 
    .DO       (spi_so_io    ), 
    .WPn      (spi_wp_n_io  ), 
    .HOLDn    (spi_hold_n_io), 
    .RESETn   (rst_n        )
);
          
endmodule
